<?php

namespace Goldenscarab\Modulus\Helpers;

use Illuminate\Support\Facades\Auth;

/**
 * Constuction de tableau de paramétrage pour constuction dynamique de boutons
 *
 * $can_prefix = 'test';
 * $url_prefix = url('home');
 *
 *
 * $am = new ActionMenu($can_prefix, $url_prefix);
 * $am->make('[UPDATE]');
 * $am->make('[DUPLICATE]');
 * $am->make('[DELETE]');
 * $am->make('[EXPORT]');
 * $am->make('[RESET-POSITION]');
 * $am->make('[SEPARATOR]');
 * $am->make('[TRUNCATE]');
 * $am->make('[READ]', ['attributes.class' => 'btn-sm btn-success text-nowrap']);
 */

class ActionMenu
{
    private $can_prefix;
    private $url_prefix;

    public function __construct($can_prefix, $url_prefix)
    {
        $this->can_prefix = $can_prefix;
        $this->url_prefix = $url_prefix;
    }

    private function can($action)
    {
        return (Auth::user())->can($this->can_prefix . '-' . $action);
    }

    public function make($alias, $attrs = [])
    {
        $default_actions = array(
            '[READ]' => [
                'label' => '<i class="fa fa-eye mr-2"></i>' . __('Consulter'),
                'href'  => ['condition' => [
                    'value' => $this->can('read'),
                    'comparator' => true,
                    'false' => '#',
                    'true'  => ['template' => [
                        'format' => '%s/read/%s',
                        'args'   => [$this->url_prefix, ':id']
                    ]],
                ]],
                'attributes' => [
                    'class' => $this->can('read') ? '' : ' disabled',
                ]
            ],
            '[SHOW]' => [
                'label' => '<i class="fa fa-eye mr-2"></i>' . __('Consulter'),
                'href'  => ['condition' => [
                    'value' => $this->can('read'),
                    'comparator' => true,
                    'false' => '#',
                    'true'  => ['template' => [
                        'format' => '%s/read/%s',
                        'args'   => [$this->url_prefix, ':id']
                    ]],
                ]],
                'attributes' => [
                    'class' => $this->can('read') ? '' : ' disabled',
                ]
            ],
            '[EDIT]' => [
                'label' => '<i class="fa fa-pencil-square-o mr-2"></i>' . __('Modifier'),
                'href'  => ['condition' => [
                    'value' => $this->can('update'),
                    'comparator' => true,
                    'false' => '#',
                    'true'  => ['template' => [
                        'format' => '%s/edit/%s',
                        'args'   => [$this->url_prefix, ':id']
                    ]],
                ]],
                'attributes' => [
                    'class' => $this->can('read') ? '' : ' disabled',
                ]
            ],
            '[UPDATE]' => [
                'label' => '<i class="fa fa-pencil-square-o mr-2"></i>' . __('Modifier'),
                'href'  => ['condition' => [
                    'value' => $this->can('update'),
                    'comparator' => true,
                    'false' => '#',
                    'true'  => ['template' => [
                        'format' => '%s/update/%s',
                        'args'   => [$this->url_prefix, ':id']
                    ]],
                ]],
                'attributes' => [
                    'class' => $this->can('read') ? '' : ' disabled',
                ]
            ],
            '[DUPLICATE]' => [
                'label' => '<i class="fa fa-clone mr-2"></i>' . __('Dupliquer'),
                'href'  => ['condition' => [
                    'value' => $this->can('create'),
                    'comparator' => true,
                    'false' => '#',
                    'true'  => ['template' => [
                        'format' => '%s/duplicate/%s',
                        'args'   => [$this->url_prefix, ':id']
                    ]],
                ]],
                'attributes' => [
                    'class' => $this->can('read') ? '' : ' disabled',
                ]
            ],
            '[DESTROY]' => [
                'label'      => '<i class="fa fa-trash mr-2"></i>' . __('Supprimer'),
                'href'       => '#',
                'attributes' => [
                    'data-href'  => ['condition' => [
                        'value' => $this->can('delete'),
                        'comparator' => true,
                        'false' => '#',
                        'true'  => ['template' => [
                            'format' => '%s/destroy/%s',
                            'args'   => [$this->url_prefix, ':id']
                        ]],
                    ]],
                    'class' => $this->can('read') ? 'text-danger delete' : ' disabled',
                ]
            ],
            '[CREATE]' => [
                'label' => '<i class="fa fa-plus mr-2"></i>' . __('Ajouter'),
                'href'  => ['condition' => [
                    'value' => $this->can('create'),
                    'comparator' => true,
                    'false' => '#',
                    'true'  => ['template' => [
                        'format' => '%s/create',
                        'args'   => [$this->url_prefix]
                    ]],
                ]],
                'attributes' => [
                    'class' => $this->can('read') ? '' : ' disabled',
                ]
            ],
            '[ADD]' => [
                'label' => '<i class="fa fa-plus mr-2"></i>' . __('Ajouter'),
                'href'  => ['condition' => [
                    'value' => $this->can('create'),
                    'comparator' => true,
                    'false' => '#',
                    'true'  => ['template' => [
                        'format' => '%s/create',
                        'args'   => [$this->url_prefix]
                    ]],
                ]],
                'attributes' => [
                    'class' => $this->can('read') ? '' : ' disabled',
                ]
            ],
            '[EXPORT]' => [
                'label' => '<i class="fa fa-file-excel-o mr-2"></i>' . __('Exporter'),
                'href'  => ['condition' => [
                    'value' => $this->can('read'),
                    'comparator' => true,
                    'false' => '#',
                    'true'  => ['template' => [
                        'format' => '%s/export',
                        'args'   => [$this->url_prefix]
                    ]],
                ]],
                'attributes' => [
                    'class' => $this->can('read') ? '' : ' disabled',
                ]
            ],
            '[TRUNCATE]' => [
                'label'      => '<i class="fa fa-times mr-2"></i>' . __('Tout supprimer'),
                'href'       => '#',
                'attributes' => [
                    'data-href'  => ['condition' => [
                        'value' => $this->can('delete'),
                        'comparator' => true,
                        'false' => '#',
                        'true'  => ['template' => [
                            'format' => '%s/truncate',
                            'args'   => [$this->url_prefix]
                        ]],
                    ]],
                    'class' => $this->can('read') ? 'text-danger confirm' : ' disabled',
                ]
            ],
            '[RESET-POSITION]' => [
                'label'      => '<i class="fa fa-list-ol mr-2"></i>' . __('Reset position'),
                'href'       => '#',
                'attributes' => [
                    'data-href'  => ['condition' => [
                        'value' => $this->can('update'),
                        'comparator' => true,
                        'false' => '#',
                        'true'  => ['template' => [
                            'format' => '%s/reset-position',
                            'args'   => [$this->url_prefix]
                        ]],
                    ]],
                    'class' => $this->can('read') ? 'confirm' : ' disabled',
                ]
            ],
            '[SEPARATOR]' => '<div class="dropdown-divider"></div>'
        );

        if (array_key_exists($alias, $default_actions)) {
            $item = $default_actions[$alias];

            // Mise à jour de l'élément avant son retour
            foreach ($attrs as $key => $value) {
                $item = data_set($item, $key, $value);
            }

            return $item;
        } else {
            throw new \InvalidArgumentException("Unknow alias name : " . $alias);
        }
    }
}
